@@ -31,8 +31,6 @@
 
 /* This is between GTK_PRIORITY_RESIZE (+10) and GTK_PRIORITY_REDRAW (+20) */
 #define META_PRIORITY_RESIZE    (G_PRIORITY_HIGH_IDLE + 15)
-/* lower than GTK_PRIORITY_REDRAW */
-#define META_PRIORITY_COMPOSITE (G_PRIORITY_HIGH_IDLE + 25)
 
 typedef struct _MetaUI MetaUI;
 
@@ -42,7 +40,7 @@ typedef gboolean (* MetaEventFunc) (XEvent *xevent, gpointer data);
 
 void meta_ui_init (int *argc, char ***argv);
 
-Display* meta_ui_get_display       (const char *name);
+Display* meta_ui_get_display (void);
 
 void meta_ui_add_event_func    (Display       *xdisplay,
                                 MetaEventFunc  func,
